local lg = love.graphics

-- backward compatible
local setMode = love.graphics.setMode or love.window.setMode
local getMode = love.graphics.getMode or love.window.getMode

res   = require 'res'
gw,gh = 320,240
sw,sh = 720,400
mode  = 'fit'
font  = lg.newFont(16)

res.set(mode,gw,gh,sw,sh)
lg.setFont(font)
setMode(sw,sh)

function love.draw()
	res.render(draw)
end

local typed = {''}
love.keyboard.setKeyRepeat(0.1,0.05)
function love.keypressed(k)
	if k == '`' then
		editmode = not editmode
		if not editmode then
			typed = {''}
		end
	end
	if k == 'escape' then
		if lg.toggleFullscreen then lg.toggleFullscreen()
		else love.window.toggleFullscreen() end
	elseif k == 'tab' then
		if mode == 'center' then
			mode = 'fit'
		elseif mode == 'fit' then
			mode = 'stretch'
		elseif mode == 'stretch' then
			mode = 'nearest'
		else
			mode = 'center'
		end
		res.set(mode,gw,gh,sw,sh)
	elseif editmode then
		if k == 'backspace' then
			table.remove(typed)		
		elseif k ~= '`' then
			table.insert(typed,k)
			if k == 'return' then
				local res_string    = table.concat(typed)
				local new_sw,new_sh = res_string:match('(%d+)x(%d+)')
				new_sw,new_sh       = tonumber(new_sw),tonumber(new_sh)
				if new_sw and new_sh then 
					local _,_,fullscreen = getMode()
					setMode(new_sw,new_sh,fullscreen)
					sw,sh = new_sw,new_sh
					res.set(mode,gw,gh,sw,sh)
				end
				typed    = {''}
				editmode = false
			end
		end
	end
end

function draw()
	-- circle @ mouse position	
	local mx,my = res.gamePosition(love.mouse.getPosition())
	lg.circle('fill',mx,my,10)
	
	-- visual indicators
	lg.rectangle('line',0,0,gw,gh)
	lg.circle('fill',gw,gh,20)
	lg.circle('fill',gw,0,20)
	lg.circle('fill',0,gh,20)
	lg.circle('fill',0,0,20)
	lg.line(0,0,gw,gh)
	lg.line(0,gh,gw,0)
	
	local t = {
		(editmode and ':' or '')..table.concat(typed),
		'reference resolution: '..gw..'x'..gh,
		'screen resolution: '..sw..'x'..sh,
		string.format( 'Mouse real position: %d,%d' , res.screenPosition(mx,my)),
		'mode: '..mode,
		'Press escape/tab to toggle fullscreen/mode',
		'Press ` to toggle resolution input (format: widthxheight)',
		'Press enter to apply the resolution',
	}
	
	lg.printf(table.concat(t,'\n'),0,0,gw,'center')
end